import { SendTransactionError, } from '@solana/web3.js';
import { GuestIdentityDriver, BundlrStorageDriver } from "./drivers";
import { TransactionBuilder, getSignerHistogram, } from "./shared";
import { nftPlugin } from "./modules";
export class Metaplex {
    /** The connection object from Solana's SDK. */
    connection;
    /** Options that dictate how to interact with the Metaplex SDK. */
    options;
    /** Encapsulates the identity of the users interacting with the SDK. */
    identityDriver;
    /** Encapsulates where assets should be uploaded. */
    storageDriver;
    /** The registered handlers for read/write operations. */
    operationHandlers = new Map();
    constructor(connection, options = {}) {
        this.connection = connection;
        this.options = options;
        this.identityDriver = new GuestIdentityDriver(this);
        this.storageDriver = new BundlrStorageDriver(this);
        this.registerDefaultPlugins();
    }
    static make(connection, options = {}) {
        return new this(connection, options);
    }
    registerDefaultPlugins() {
        this.use(nftPlugin());
    }
    use(plugin) {
        plugin.install(this);
        return this;
    }
    identity() {
        return this.identityDriver;
    }
    setIdentity(identity) {
        this.identityDriver = identity;
        return this;
    }
    storage() {
        return this.storageDriver;
    }
    setStorage(storage) {
        this.storageDriver = storage;
        return this;
    }
    async sendTransaction(transaction, signers = [], sendOptions = {}) {
        if (transaction instanceof TransactionBuilder) {
            signers = [...transaction.getSigners(), ...signers];
            transaction = transaction.toTransaction();
        }
        const { keypairs, identities } = getSignerHistogram(signers);
        for (let i = 0; i < identities.length; i++) {
            if (!identities[i].is(this.identity())) {
                await identities[i].signTransaction(transaction);
            }
        }
        return this.identity().sendTransaction(transaction, keypairs, sendOptions);
    }
    async confirmTransaction(signature, commitment) {
        const rpcResponse = await this.connection.confirmTransaction(signature, commitment);
        let transaction_error = rpcResponse.value.err;
        if (transaction_error) {
            // TODO: Custom errors.
            throw new SendTransactionError(`Transaction ${signature} failed (${JSON.stringify(transaction_error)})`, [transaction_error.toString()]);
        }
        return rpcResponse;
    }
    async sendAndConfirmTransaction(transaction, signers, confirmOptions) {
        const signature = await this.sendTransaction(transaction, signers, confirmOptions);
        await this.confirmTransaction(signature, confirmOptions?.commitment);
        return signature;
    }
    async getAccountInfo(publicKey, commitment) {
        return this.connection.getAccountInfo(publicKey, commitment);
    }
    async getMultipleAccountsInfo(publicKeys, commitment) {
        const accounts = await this.connection.getMultipleAccountsInfo(publicKeys, commitment);
        return accounts;
    }
    register(operation, operationHandler) {
        this.operationHandlers.set(operation, operationHandler);
        return this;
    }
    async execute(operation, confirmOptions) {
        const operationHandler = this.operationHandlers.get(operation.constructor);
        if (!operationHandler) {
            // TODO: Custom errors.
            throw new SendTransactionError(`No operation handler registered for ${operation.constructor.name}`);
        }
        const handler = new operationHandler(this, confirmOptions);
        return handler.handle(operation);
    }
}
//# sourceMappingURL=Metaplex.js.map