export class Loader {
    status = 'pending';
    result;
    error;
    abortSignal;
    setAbortSignal(abortSignal) {
        this.abortSignal = abortSignal;
        return this;
    }
    async reload(metaplex, options = {}) {
        if (this.isLoading()) {
            // TODO: Custom errors.
            throw new Error('Loader is already running.');
        }
        // Prepare abort listener.
        const abortListener = (reason) => {
            this.status = 'canceled';
            this.error = reason;
        };
        this.abortSignal?.addEventListener('abort', abortListener, { once: true });
        try {
            // Start loading.
            this.status = 'running';
            this.error = undefined;
            this.result = await this.handle(metaplex);
            // Mark as successful if the loader wasn't aborted.
            if (!this.wasCanceled()) {
                this.status = 'successful';
            }
            // Return the loaded result.
            return this.result;
        }
        catch (error) {
            // Capture the error and reset the result.
            this.error = error;
            this.result = undefined;
            // Mark as failed if the loader wasn't aborted.
            if (!this.wasCanceled()) {
                this.status = 'failed';
            }
            // Return undefined result if loaded aborted or if we want to fail silently.
            if (this.wasCanceled() || (options.failSilently ?? false)) {
                return this.result;
            }
            // Otherwise, re-throw the error.
            throw error;
        }
        finally {
            // Clean up the abort listener.
            this.abortSignal?.removeEventListener('abort', abortListener);
        }
    }
    async load(metaplex, options = {}) {
        if (!this.isPending()) {
            return this.result;
        }
        return this.reload(metaplex, options);
    }
    reset() {
        this.status = 'pending';
        this.result = undefined;
        this.error = undefined;
        return this;
    }
    loadWith(preloadedResult) {
        this.status = 'successful';
        this.result = preloadedResult;
        this.error = undefined;
        return this;
    }
    getStatus() {
        return this.status;
    }
    getResult() {
        return this.result;
    }
    getError() {
        return this.error;
    }
    isPending() {
        return this.status === 'pending';
    }
    isLoading() {
        return this.status === 'running';
    }
    isLoaded() {
        return this.status !== 'pending' && this.status !== 'running';
    }
    wasSuccessful() {
        return this.status === 'successful';
    }
    wasFailed() {
        return this.status === 'failed';
    }
    wasCanceled() {
        return this.status === 'canceled';
    }
}
//# sourceMappingURL=Loader.js.map